@@ -59,6 +59,12 @@ class Field:
     def __getattr__(self, name):
         return getattr(self.field_pb, name)
 
+    @property
+    def name(self) -> str:
+        """Used to prevent collisions with python keywords"""
+        name = self.field_pb.name
+        return name + "_" if name in utils.RESERVED_NAMES else name
+
     @utils.cached_property
     def ident(self) -> metadata.FieldIdentifier:
         """Return the identifier to be used in templates."""
@@ -239,6 +245,15 @@ class MessageType:
         # Identity is sufficiently unambiguous.
         return hash(self.ident)
 
+    def oneof_fields(self, include_optional=False):
+        oneof_fields = collections.defaultdict(list)
+        for field in self.fields.values():
+            # Only include proto3 optional oneofs if explicitly looked for.
+            if field.oneof and not field.proto3_optional or include_optional:
+                oneof_fields[field.oneof].append(field)
+
+        return oneof_fields
+
     @utils.cached_property
     def field_types(self) -> Sequence[Union['MessageType', 'EnumType']]:
         answer = tuple(
@@ -583,6 +598,15 @@ class Method:
     def client_output_async(self):
         return self._client_output(enable_asyncio=True)
 
+    def flattened_oneof_fields(self, include_optional=False):
+        oneof_fields = collections.defaultdict(list)
+        for field in self.flattened_fields.values():
+            # Only include proto3 optional oneofs if explicitly looked for.
+            if field.oneof and not field.proto3_optional or include_optional:
+                oneof_fields[field.oneof].append(field)
+
+        return oneof_fields
+
     def _client_output(self, enable_asyncio: bool):
         """Return the output from the client layer.
 
@@ -685,6 +709,10 @@ class Method:
 
         return answer
 
+    @utils.cached_property
+    def flattened_field_to_key(self):
+        return {field.name: key for key, field in self.flattened_fields.items()}
+
     @utils.cached_property
     def legacy_flattened_fields(self) -> Mapping[str, Field]:
         """Return the legacy flattening interface: top level fields only,
@@ -731,7 +759,7 @@ class Method:
 
         # Return the first repeated field.
         for field in self.output.fields.values():
-            if field.repeated and field.message:
+            if field.repeated:
                 return field
 
         # We found no repeated fields. Return None.
@@ -773,7 +801,7 @@ class Method:
 
         # If this message paginates its responses, it is possible
         # that the individual result messages reside in a different module.
-        if self.paged_result_field:
+        if self.paged_result_field and self.paged_result_field.message:
             answer.append(self.paged_result_field.message)
 
         # Done; return the answer.
